@@ -5,16 +5,27 @@ use strict;
 use Distributed::Process;
 import Distributed::Process;
 
+use Thread::Queue;
 use Distributed::Process::Interface;
 use Distributed::Process::BaseWorker;
 our @ISA = qw/ Distributed::Process::BaseWorker Distributed::Process::Interface /;
 
+sub new {
+
+    my $class = shift;
+    my $self = $class->SUPER::new(@_);
+    $self->message_queue();
+
+    $self;
+}
+
 sub command_handlers {
 
     my $self = shift;
     return @{$self->{_command_handlers} ||= [
 	$self->SUPER::command_handlers(),
 	[ qr|^/synchro|, sub { my $tok = (split /\s+/, $_[0])[1]; chomp $tok; $self->synchro_received($tok) } ],
+	[ qr|^/begin_return|, sub { $self->begin_return() }, 'begin_return' ],
 	[ qr|^/begin_results|, sub { $self->begin_results() }, 'begin_results' ],
         [ qr|^/worker|, sub { $self->id((split /\s+/, $_[0])[1]); $self->master()->worker_ready($self) } ],
     ]};
@@ -31,7 +42,9 @@ sub synchro {
     my $self = shift;
     my $token = shift;
 
+    INFO "synchro $token";
     $self->send("/synchro $token");
+    $self->master()->master_worker()->synchro($token);
 }
 
 sub synchro_received {
@@ -39,6 +52,7 @@ sub synchro_received {
     my $self = shift;
     my $token = shift;
     $self->master()->synchro_received($self, $token);
+    return;
 }
 
 sub _run_code_in_args {
@@ -56,6 +70,12 @@ sub _run_code_in_args {
     @arg;
 }
 
+sub message_queue {
+
+    my $self = shift;
+    $self->{_message_queue} ||= new Thread::Queue;
+}
+
 sub go_remote {
 
     my $self = shift;
@@ -73,7 +93,22 @@ sub go_remote {
 		    my $s = shift;
 		    local $" = " ";
 		    my @arg = $s->_run_code_in_args(@_);
-		    $s->send("/run $name @arg");
+		    if ( !defined(wantarray) ) {
+			DEBUG "running $name in void context";
+			$s->send("/run $name @arg");
+			return '';
+		    }
+		    else {
+			DEBUG "running $name in non void context";
+			my $q = $s->message_queue();
+			$s->send("/run_with_return $name @arg");
+			my @res;
+			while ( my $v = $q->dequeue() ) {
+			    last if $v eq '/end_return';
+			    push @res, $v;
+			}
+			return wantarray ? @res : $res[0];
+		    }
 		};
 	    }
 	    if ( $name =~ /::$/ ) {
@@ -102,6 +137,19 @@ sub get_result {
     $self->send("/get_result");
 }
 
+sub begin_return {
+
+    my $self = shift;
+
+    $self->command_handlers();
+    unshift @{$self->{_command_handlers}}, [
+	qr/.*/,
+	sub { $self->handle_return(@_) },
+	'return_value',
+    ];
+    return;
+}
+
 sub begin_results {
 
     my $self = shift;
@@ -132,6 +180,17 @@ sub handle_result {
     return;
 }
 
+sub handle_return {
+
+    my $self = shift;
+    DEBUG "Handling returned value '@_'";
+    $self->message_queue()->enqueue(@_);
+    if ( $_[0] =~ m|^/end_return| ) {
+	shift @{$self->{_command_handlers}};
+    }
+    return;
+}
+
 sub time {
 
     my $self = shift;
@@ -146,6 +205,20 @@ sub is_ready {
     return defined($self->id());
 }
 
+sub worker_index {
+
+    my $self = shift;
+    $self->master()->worker_index($self);
+}
+sub postpone {
+
+    my $self = shift;
+    my $method = shift;
+
+    select undef, undef, undef, $self->worker_index() * 1 / ($self->master()->frequency() || 1);
+    $self->$method(@_);
+}
+
 foreach my $method ( qw/ id master / ) {
 
     no strict 'refs';
@@ -158,4 +231,3 @@ foreach my $method ( qw/ id master / ) {
 }
 
 1;
-1;
